package com.muyu.quest.service.impl;

import java.util.*;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.quest.domain.*;
import com.muyu.quest.manager.TaskManager;
import com.muyu.quest.model.DataModel;
import com.muyu.quest.model.DataValueModel;
import com.muyu.quest.exception.TaskException;
import com.muyu.quest.remote.RemoteDataSourceService;
import com.muyu.quest.req.NodeReq;
import com.muyu.quest.req.TaskReq;
import com.muyu.quest.resp.TaskResp;
import com.muyu.quest.service.INodeDispositionService;
import com.muyu.quest.service.INodeService;
import com.muyu.quest.service.ITaskExportService;
import com.muyu.quest.utils.NodeUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import com.muyu.quest.mapper.TaskMapper;
import com.muyu.quest.service.TaskService;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;

/**
 * 任务Service业务层处理
 *
 * @Author: 胡杨
 * @date 2024-08-22
 */
@Slf4j
@Service
public class TaskServiceImpl extends ServiceImpl<TaskMapper, Task>
        implements TaskService {

    @Resource
    private INodeService nodeService;
    @Resource
    private INodeDispositionService dispositionService;
    @Resource
    private TaskMapper taskMapper;
    @Resource
    private RemoteDataSourceService remoteDataSourceService;
    @Resource
    private ITaskExportService taskExportService;

    private final static TaskManager taskManager = TaskManager.getTaskManager();

    /**
     * 查询任务
     *
     * @param taskCode 任务编码
     * @return 任务
     */
    @Override
    public Task selectTaskByTaskCode(String taskCode)
    {
        LambdaQueryWrapper<Task> queryWrapper = new LambdaQueryWrapper<>();
        Assert.notNull(taskCode, "taskCode不可为空");
        queryWrapper.eq(Task::getTaskCode, taskCode);
        return this.getOne(queryWrapper);
    }

    /**
     * 查询任务
     *
     * @param id 任务主键
     * @return 任务
     */
    @Override
    public Task selectTaskById(Long id)
    {
        LambdaQueryWrapper<Task> queryWrapper = new LambdaQueryWrapper<>();
        Assert.notNull(id, "id不可为空");
        queryWrapper.eq(Task::getId, id);
        return this.getOne(queryWrapper);
    }

    /**
     * 唯一编码判断
     * @param task
     * @return
     */
    @Override
    public Boolean checkTaskCodeUnique(Task task) {
        LambdaQueryWrapper<Task> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Task::getTaskCode, task.getTaskCode());
        return this.exists(queryWrapper);
    }

    @Override
    public Boolean checkIdUnique(Task task) {
        LambdaQueryWrapper<Task> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Task::getId, task.getId());
        return this.exists(queryWrapper);
    }

    @Override
    public void removeBatch(List<Long> list) {
        this.removeBatchByIds(list);
    }

    /**
     * 查询任务列表
     *
     * @param taskReq 任务
     * @return 任务
     */
    @Override
    public List<TaskResp> selectTaskList(TaskReq taskReq)
    {
        LambdaQueryWrapper<Task> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(taskReq.getTaskName())){
            queryWrapper.like(Task::getTaskName, taskReq.getTaskName());
        }
        if (StringUtils.isNotEmpty(taskReq.getState())){
            queryWrapper.eq(Task::getState, taskReq.getState());
        }
        if (StringUtils.isNotEmpty(taskReq.getTaskType())){
            queryWrapper.eq(Task::getTaskType, taskReq.getTaskType());
        }
        return this.list(queryWrapper)
                .stream()
                .map(TaskResp::build)
                .toList();
    }

    // 查询节点所有类型
    public List<NodeType> selectNodeTypeList(){
        return taskMapper.selectNodeTypeList();
    }

    /**
     * //TODO 执行任务
     * 获取任务所有节点配置信息 拼接sql语句
     * @param taskCode 任务编码
     * @return 成功与否
     */
    @Override
    public String execute(String taskCode) {
        log.info("任务编码 {} 开始执行......",taskCode);
        // 查询任务所有节点
        List<Node> nodeListAll = nodeService.selectNodeList(new NodeReq().buildTaskCode(taskCode));
        // 节点初始化
        HashMap<String, List<Node>> nodeMap = nodeCheckNorm(nodeListAll);
        // 获取查询SQL
        String findSql = getFindSql(nodeMap);
        // 获取查询条数
        int count = getFindCount(findSql, nodeMap);
        // 划分线程 每次查询10000条
        int pageSize = 10000;
        int threadNum = count / pageSize + 1;
        log.info("任务 {} 总共需要 {} 条数据, 划分为线程{}条",taskCode,count,threadNum);
        for (int i = 0; i < threadNum; i++) {
            int index = i+1;
            // 添加进入任务队列
            taskManager.execute(() -> {
                String exportCode = UUID.randomUUID().toString().replace("-","");
                // 获取新SQL 并执行
                String sql = findSql + " LIMIT "+pageSize+" OFFSET "+(index-1)*pageSize;
                String addSql = getAddSql(nodeMap, sql);
                int addSqlMaxLength = Math.min(addSql.length(), 30000);
                TaskExport entity = new TaskExport(taskCode,exportCode, addSql.substring(0,addSqlMaxLength), 0, "");
                taskExportService.save(entity);
                Result addResult = remoteDataSourceService.addTableValue(new DataValueModel(4L, addSql));
                log.info("任务 {} 第 {} 线程执行结果 {}",taskCode,index,addResult);
                if (addResult.getCode() != 200){
                    int errorMaxLength = Math.min(addResult.getMsg().length(), 30000);
                    entity.setError(addResult.getMsg().substring(0,errorMaxLength));
                    entity.setStart(2);
                }else {
                    entity.setStart(1);
                }
                taskExportService.updateByExportCode(entity);
            });
        }

        return "执行成功";
    }



    @Override
    public String testExecute(List<Node> nodeListAll) {
        log.info("开始测试执行......");
        // 查询节点类型与其规范
        List<NodeType> nodeTypeList = selectNodeTypeList();
        /* 节点组成校验 */
        NodeUtils.nodeCheckMakeUp(nodeListAll, nodeTypeList);
        /* 节点连接规范校验 */
        NodeUtils.nodeCheckNorm(nodeListAll, nodeTypeList);

        return "测试成功,无异常";
    }


    /**
     * 执行任务第一步: 校验节点规范 并返回初始化节点map
     */
    private HashMap<String, List<Node>> nodeCheckNorm(List<Node> nodeListAll) {
        // 查询节点类型与其校验相关配置
        List<NodeType> nodeTypeList = selectNodeTypeList();
        /* 节点组成校验 */
        NodeUtils.nodeCheckMakeUp(nodeListAll, nodeTypeList);
        /* 节点连接规范校验 */
        NodeUtils.nodeCheckNorm(nodeListAll, nodeTypeList);

        return NodeUtils.nodeInit(nodeListAll);
    }

    /**
     * 执行任务第二步: 拼接查询SQL
     */
    private String getFindSql(HashMap<String, List<Node>> nodeMap) {
        String findSql = "";
        List<Node> uniteNodes = nodeMap.get("unite");
        if (uniteNodes==null || uniteNodes.isEmpty()){
            Node tableNode = nodeMap.get("table").get(0);
            List<NodeDisposition> dispList = getNodeDisp(tableNode);
            findSql = NodeUtils.tableNode(dispList);
        }else{
            Node uniteNode = uniteNodes.get(0);
            List<NodeDisposition> dispList = getNodeDisp(uniteNode);
            findSql = NodeUtils.nodeDispUnite(dispList);
        }
        return findSql;
    }

    /**
     * 执行任务第三步: 获取查询条数
     */
    private int getFindCount(String findSql, HashMap<String, List<Node>> nodeMap) {
        System.out.println(findSql);
        String findCountSql = "";
        List<Node> uniteNodes = nodeMap.get("unite");
        findSql = findSql.replace("  "," ");
        String[] s1 = findSql.split(" ");
        if (uniteNodes!=null && !uniteNodes.isEmpty()){
            s1[1] = "COUNT(" + s1[1].split(",")[0]+ ")";
            findCountSql = StringUtils.join(s1," ");
        }else {

            String[] split = s1[s1.length-1].split("\\.");
            findCountSql = "SELECT TABLE_ROWS " +
                    "FROM INFORMATION_SCHEMA.TABLES " +
                    "WHERE TABLE_SCHEMA = "+split[0]+" AND TABLE_NAME = '"+split[1]+"';";
            findCountSql = findCountSql.replace("`","'");
        }
        System.out.println(findCountSql);
        Result tableValue = remoteDataSourceService.getTableValueTotal(new DataValueModel(4L, findCountSql));
        System.out.println(tableValue);

        return (int) tableValue.getData();
    }

    /**
     * 获取新增SQL
     */
    private String getAddSql(HashMap<String, List<Node>> nodeMap, String findSql) {
        Result<List<List<DataModel>>> tableValue = remoteDataSourceService.findTableValue(new DataValueModel(4L, findSql));
        if (tableValue.getCode() != 200){
            throw new TaskException(tableValue.getMsg());
        }
        List<List<DataModel>> data = tableValue.getData();


        List<NodeDisposition> dispList = getNodeDisp(nodeMap.get("exportation").get(0));
        return NodeUtils.nodeDispExportation(dispList, data);

    }

    // 查询节点配置信息
    public List<NodeDisposition> getNodeDisp(Node node){
        List<NodeDisposition> dispList = dispositionService
                .selectNodeDispositionList(new NodeDisposition().buildNodeCode(node.getNodeCode()));
        if (dispList.isEmpty()){
            throw new TaskException("节点 "+node+" 配置为空");
        }
        return dispList;
    }


}
